//******************************************************************************
// "RADIOSITY LIGHTMAPS" by Georgy Moshkin
// -----------------------------------------------
// email  : tmtlib@narod.ru
// WWW    : http://www.tmtlib.narod.ru/
//
// Version: 0.3
// Example: 8
//
// License:
// --------
// This unit is freely distributable without licensing fees and is
// provided without guarantee or warrantee expressed or implied. This unit
// is Public Domain. Feel free to use or enhance this code. You can use the
// program for any purpose you see fit.
//
//******************************************************************************
unit rad8form;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls,
  OpenGL, //
  rad8;   // OpenGL radiosity-v8 unit


type
  TForm1 = class(TForm)
    Panel1: TPanel;
    Panel2: TPanel;
    Panel3: TPanel;
    Timer1: TTimer;
    ScrollBar5: TScrollBar;
    Memo1: TMemo;

    procedure FormCreate(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure TimerDraw(Sender: TObject);
  private
    { Private declarations }
    rc : HGLRC;    // RC
    dc  : HDC;     // DC

    procedure InitGL;
    procedure glDraw;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////

procedure TForm1.glDraw(); //     OPENGL 
var i,j:integer;
begin

 //       ,   rad8.pas

 // LIGHT SOURCE:
 lmap[1].FillRect(40,40,40,40,255,255,255); //  

 // LIGHTMAP GENERATION
 if rpx then
  DistributeLight; //    
                   //
                   //  
                   //   for i:=0 to ... do DistributeLight;

 // OPENGL TEXTURE GENERATION
 for i:=0 to leveldim-1 do
  lmap[i].UpdateTexture(GL_LINEAR); //  OpenGL-   


 glClearColor(0,0,0, 1.0); 	   //  
 glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT); //  
 glLoadIdentity(); //    

 glViewport(0, 0, Panel3.Width, Panel3.Height); // 
 // viewport   OpenGL- 

 //    
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(45.0, Panel3.Width/Panel3.Height, 0.1, 500.0);
  glMatrixMode(GL_MODELVIEW);



 //  
  gluLookAt(0,0,-100, //  
            0,0,0,    //  
            0,1,0);    //  

 //       ScrollBar-
  glRotatef(360*ScrollBar5.Position/100,0,1,0);

  renderQuad(0); //   
  renderQuad(1); //   ()

 SwapBuffers(DC);

end;


//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
procedure TForm1.InitGL;
begin
  glClearColor(0.0, 0.0, 0.0, 1.0); 	   // White Background
  glShadeModel(GL_SMOOTH);                 // Enables Smooth Color Shading
  glClearDepth(1.0);                       // Depth Buffer Setup
  glEnable(GL_DEPTH_TEST);                 // Enable Depth Buffer
  glDepthFunc(GL_LESS);		           // The Type Of Depth Test To Do

  glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);

  //      :
  glShadeModel(GL_FLAT);
  glHint(GL_FOG_HINT, GL_FASTEST);
  glHint(GL_LINE_SMOOTH_HINT, GL_FASTEST);
  glHint(GL_POINT_SMOOTH_HINT, GL_FASTEST);
  glHint(GL_POLYGON_SMOOTH_HINT, GL_FASTEST);



end;

//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////

procedure TForm1.FormCreate(Sender: TObject);
var pfd : TPIXELFORMATDESCRIPTOR;
    pf  : Integer;
begin
  DecimalSeparator:='.';
  dc:=GetDC(Panel3.Handle);

  pfd.nSize:=sizeof(pfd);
  pfd.nVersion:=1;
  pfd.dwFlags:=PFD_DRAW_TO_WINDOW or PFD_SUPPORT_OPENGL or PFD_DOUBLEBUFFER or 0;
  pfd.iPixelType:=PFD_TYPE_RGBA;
  pfd.cColorBits:=32;

//pfd.cDepthBits:=16; //?

  pf :=ChoosePixelFormat(dc, @pfd);
  SetPixelFormat(dc, pf, @pfd);

  rc :=wglCreateContext(dc);    // RC
  wglMakeCurrent(dc,rc);        // DC

  InitGL;

  randomize;

  MakeRoom; //  3d  

  rptrmake(256,256); //     radiosity

  ScrollBar5.SetParams(0,0,100); //   ScrollBar-
end;

//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////

procedure TForm1.FormResize(Sender: TObject);
begin
  glViewport(0, 0, Panel3.Width, Panel3.Height);


  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(45.0, Panel3.Width/Panel3.Height, 0.1, 500.0);

  glMatrixMode(GL_MODELVIEW);



  glDraw();
end;

//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////

procedure TForm1.FormDestroy(Sender: TObject);
begin
  wglMakeCurrent(0,0);
  wglDeleteContext(rc);
end;

procedure TForm1.TimerDraw(Sender: TObject);
begin
glDraw();
end;

end.

